package org.apache.bcel.generic;

import org.apache.bcel.util.ByteSequence;
import java.io.IOException;
import java.io.DataOutputStream;

public class IINC extends LocalVariableInstruction
{
    private boolean wide;
    private int c;
    
    IINC() {
    }
    
    public IINC(final int n, final int c) {
        super.setOpcode((short)132);
        super.setLength(3);
        this.setIndex(n);
        this.setIncrement(c);
    }
    
    @Override
    public void accept(final Visitor v) {
        v.visitLocalVariableInstruction(this);
        v.visitIINC(this);
    }
    
    @Override
    public void dump(final DataOutputStream out) throws IOException {
        if (this.wide) {
            out.writeByte(196);
        }
        out.writeByte(super.getOpcode());
        if (this.wide) {
            out.writeShort(super.getIndex());
            out.writeShort(this.c);
        }
        else {
            out.writeByte(super.getIndex());
            out.writeByte(this.c);
        }
    }
    
    public final int getIncrement() {
        return this.c;
    }
    
    @Override
    public Type getType(final ConstantPoolGen cp) {
        return Type.INT;
    }
    
    @Override
    protected void initFromFile(final ByteSequence bytes, final boolean wide) throws IOException {
        this.wide = wide;
        if (wide) {
            super.setLength(6);
            super.setIndexOnly(bytes.readUnsignedShort());
            this.c = bytes.readShort();
        }
        else {
            super.setLength(3);
            super.setIndexOnly(bytes.readUnsignedByte());
            this.c = bytes.readByte();
        }
    }
    
    public final void setIncrement(final int c) {
        this.c = c;
        this.setWide();
    }
    
    @Override
    public final void setIndex(final int n) {
        if (n < 0) {
            throw new ClassGenException("Negative index value: " + n);
        }
        super.setIndexOnly(n);
        this.setWide();
    }
    
    private void setWide() {
        this.wide = (super.getIndex() > 255);
        if (this.c > 0) {
            this.wide = (this.wide || this.c > 127);
        }
        else {
            this.wide = (this.wide || this.c < -128);
        }
        if (this.wide) {
            super.setLength(6);
        }
        else {
            super.setLength(3);
        }
    }
    
    @Override
    public String toString(final boolean verbose) {
        return super.toString(verbose) + " " + this.c;
    }
}
